/*! 
 \file camera_01.js
 \author Martin Blaha, Michal Krec, Tadeas Lejsek, Petr Marek, Tomas Nouza
 \brief Camera
 \details Initialization of camera GUI, button listeners
 */
 
/*! \brief <a href="camera__01_8js_source.html">Camera</a> */
/* ===== Camera ===== */
function Camera(connection, ip_adress, camera_div) {
    var data;
    var table_topics;
    var table_properties;
    var button_topic;
    var button_properties;
    var camera_pic;
    var isEmpty = true;

    var topicName;
    var lastPic;
    var matrix_properties_name = ['width', 'height', 'quality'];
    var matrix_properties_value = ['640', '480', '50'];

    /*! \brief <a href="camera__01_8js_source.html">Initialization of camera GUI</a> */
    /* ===== Initialization of camera GUI ===== */
    this.init = function init() {

        // Remove previous childnodes 
        while (camera_div.hasChildNodes()) {
            camera_div.removeChild(camera_div.lastChild);
        }
        var nameDiv = document.createElement("p");
        var nameText = document.createTextNode('Camera');
        nameDiv.appendChild(nameText);
        camera_div.appendChild(nameDiv);

        // Add Select topic button
        button_topic = document.createElement("input");
        button_topic.type = "submit";
        button_topic.id = 'camera_topic';
        button_topic.value = 'Select topic';
        camera_div.appendChild(button_topic);

        // Add Properties button
        button_properties = document.createElement("input");
        button_properties.type = "submit";
        button_properties.id = 'camera_prop';
        button_properties.value = 'Properties';
        camera_div.appendChild(button_properties);

        // Add camera picture
        camera_pic = document.createElement("div");
        camera_pic.id = 'camera_pic';
        camera_div.appendChild(camera_pic);

        // Add event lisener
        button_topic.addEventListener('click', change_cam, false);
        button_properties.addEventListener('click', change_prop, false);
    };

    /*! \brief <a href="camera__01_8js_source.html">Button listener for changing properties</a> */
    /* ===== Button listener for changing properties ===== */
    var change_prop = function(e) {

        // Remove previous childnodes 
        while (camera_pic.hasChildNodes()) {
            camera_pic.removeChild(camera_pic.lastChild);
        }

        // Table with properties
        table_properties = document.createElement("table"); // Add table
        table_properties.border = "1";
        var tbody = document.createElement("tbody"); // Add body
        table_properties.appendChild(tbody);
        for (var j = 0; j < matrix_properties_name.length; j++) { // Count of rows
            var row = document.createElement("tr"); // Add row
            for (var k = 0; k < 2; k++) { // Count of columns
                var cell = document.createElement("td"); // Add cell
                if (k === 0) { // Name of property
                    data = document.createTextNode(matrix_properties_name[j]);
                }
                if (k === 1) { // Value of property
                    data = document.createElement("input");
                    data.type = "text";
                    data.name = "table_properties" + j;
                    data.id = "table_properties_" + matrix_properties_name[j];
                    data.value = matrix_properties_value[j];
                    camera_pic.appendChild(data);
                }
                cell.appendChild(data);
                row.appendChild(cell);
            }
            tbody.appendChild(row);
        }
        camera_pic.appendChild(table_properties);

        // OK button
        var ok_button = document.createElement("input");
        ok_button.type = "submit";
        ok_button.name = 'okProp';
        ok_button.id = 'okProp';
        ok_button.value = 'OK';
        camera_pic.appendChild(ok_button);

        ok_button.addEventListener('click', function(msg) {
            var width = document.getElementById('table_properties_width').value;
            var height = document.getElementById('table_properties_height').value;
            var quality = document.getElementById('table_properties_quality').value;

            matrix_properties_value[0] = width;
            matrix_properties_value[1] = height;
            matrix_properties_value[2] = quality;

            data = document.createElement("img");
            if (!isEmpty) {
                data.src = "http://" + ip_adress + ":8080/stream?topic=" + topicName + "?width=" + width + "?height=" + height + "?quality=" + quality;
                data.width = width;
                data.height = height;
            }

            // Remove previous childnodes 
            while (camera_pic.hasChildNodes()) {
                camera_pic.removeChild(camera_pic.lastChild);
            }

            lastPic = data;
            camera_pic.appendChild(data);
        }, false);

        // CANCEL button
        var cancel_button = document.createElement("input");
        cancel_button.type = "submit";
        cancel_button.name = 'caProp';
        cancel_button.id = 'caProp';
        cancel_button.value = 'Cancel';
        camera_pic.appendChild(cancel_button);

        cancel_button.addEventListener('click', function(msg) {
            // Remove previous childnodes 
            while (camera_pic.hasChildNodes()) {
                camera_pic.removeChild(camera_pic.lastChild);
            }

            //camera_pic.appendChild(lastPic);
        }, false);
    };

    /*! \brief <a href="camera__01_8js_source.html">Button listener for changing Image topic</a> */
    /* ===== Button listener for changing Image topic ===== */
    var change_cam = function(e) {
        getTopicImage();
    };

    /*! \brief <a href="camera__01_8js_source.html">Get topics which have type of message Image</a> */
    /* ===== Get topics which have type of message Image ===== */
    function getTopicImage() {
        var topicImage;

        // CallBackFunction
        var callBackTopic = function(msg) {
            var message = "";
            isEmpty = true;
            if (msg.topics.length != 0) {
                isEmpty = false;
                for (var i = 0; i < msg.topics.length; i++) {
                    message += msg.topics[i] + "\n";
                }
                var datasplit = message.split("\n");
                topicImage = datasplit.sort();
                select_topic_name(topicImage);
            } else {
                alert('No topics with type of message: sensor_msgs/Image ');
            }
        };

        connection.callService(callBackTopic, '/rosapi/topics_for_type', {"type": 'sensor_msgs/Image'});
    }

    /*! \brief <a href="camera__01_8js_source.html">Select topics which have type of message Image</a> */
    /* ===== Select topics which have type of message Image ===== */
    function select_topic_name(msg) {
        // Remove previous childnodes
        while (camera_pic.hasChildNodes()) {
            camera_pic.removeChild(camera_pic.lastChild);
        }

        // Table with topics
        table_topics = document.createElement("table"); // Add table
        table_topics.border = "0";
        var tbody = document.createElement("tbody"); // Add body
        table_topics.appendChild(tbody);
        for (var j = 1; j < msg.length; j++) { // Count of rows
            var row = document.createElement("tr"); // Add row
            for (var k = 0; k < 2; k++) { // Count of columns
                var cell = document.createElement("td"); // Add cell
                if (k === 1) { // Name of topic
                    data = document.createTextNode(msg[j]);
                }
                if (k === 0) { // Radiobutton of topic
                    data = document.createElement("input");
                    data.type = "radio";
                    data.name = msg[j];
                    data.id = 'topic' + j;

                    data.addEventListener('click', function(msg) {
                        topicName = this.name;
                        data = document.createElement("img");
                        if (!isEmpty) {
                            data.src = "http://" + ip_adress + ":8080/stream?topic=" + this.name + "?quality=10";
                            data.width = "640";
                            data.height = "480";
                        }
                        // Remove previous childnodes
                        while (camera_pic.hasChildNodes()) {
                            camera_pic.removeChild(camera_pic.lastChild);
                        }

                        camera_pic.appendChild(data);
                        lastPic = data;
                    }, false);
                }
                cell.appendChild(data);
                row.appendChild(cell);
            }
            tbody.appendChild(row);
        }
        camera_pic.appendChild(table_topics);
    }
}