/*! 
 \file connection_01.js
 \author Martin Blaha, Michal Krec, Tadeas Lejsek, Petr Marek, Tomas Nouza
 \brief Connection
 \details Create connection, change IP adress, advertise, unadvertise, subscribe, unsubscribe
 */
 
/*! \brief <a href="connection__01_8js_source.html">Connection</a> */
/* ===== Connection ===== */ 
function Connection() {

    var joy_o, map_o, info_o, cam_o;
    var subIP;
    var formIP;
    var connection;
    
    /*! \brief <a href="connection__01_8js_source.html">Create connection</a> */
    /* ===== Create connection ===== */ 
    function createConnection(ip_adress) {
        console.log("ROS connected to: " + ip_adress);
        return new ros.Bridge("ws://" + ip_adress + ":9090"); // Create connection to IP adress on port 9090
    }

    /*! \brief <a href="connection__01_8js_source.html">Change IP adress, connect and inicialize joystick, map, laser, info (topics, nodes, services), camera</a> */
    /* ===== Change IP adress, connect and inicialize joystick, map, laser, info (topics, nodes, services), camera ===== */
    this.ip_change = function() {
        var ip_adress = document.getElementById('ip_adr_value').value; // Get value of IP adress from HTML page
        connection = createConnection(ip_adress); // Create connection

        // Inicialization of joystick
        joy_o = new Joystick(connection, document.getElementById('joystick'));
        joy_o.init();

        // Inicialization of map and laser
        map_o = new MapaLaser(connection, document.getElementById('laser'));
        map_o.init();

        // Inicialization of info about topics, nodes, services
        info_o = new Info(connection, document.getElementById('info'));
        info_o.init();

        // Inicialization of camera
        cam_o = new Camera(connection, ip_adress, document.getElementById('camera'));
        cam_o.init();

        // Connection is openned
        connection.onOpen = function() {
            //advertise(connection);
            subscribe(connection);
           

            window.addEventListener('unload', function() {
                unsubscribe(connection);
                console.log("Closing page");
            }, false);

            subIP.value = "disconnect";
            formIP.setAttribute('action', 'javascript:con.disconnect();');

            // Terminal
            var terminal_div = document.getElementById('terminal');
            while (terminal_div.hasChildNodes()) {
                terminal_div.removeChild(terminal_div.lastChild);
            }
            // Add terminal new tab button
            var terminalButton;
            terminalButton = document.createElement("input");
            terminalButton.type = "submit";
            terminalButton.id = 'button_new_terminal';
            terminalButton.value = 'Open terminal in new Tab';
            terminal_div.appendChild(terminalButton);
            
            terminalButton.addEventListener('click', function() {
                var win = window.open("http://" + ip_adress + ":4200", '_blank');
                win.focus();
            }, false);

            var terminal = document.createElement("iframe");
            terminal.name = "iframe";
            terminal.src = "http://" + ip_adress + ":4200";
            terminal.width = "650";
            terminal.height = '400';
            terminal_div.appendChild(terminal);
        };

        // Connection is closed
        connection.onClose = function() {
            console.log("Close connection");
        };
    };

    /*! \brief <a href="connection__01_8js_source.html">Disconnect</a> */
    /* ===== Disconnect ===== */ 
    this.disconnect = function() {
        joy_o.unadvertise();
        unsubscribe(connection);
        console.log('Konec');
        subIP.value = "connect";
        formIP.setAttribute('action', 'javascript:con.ip_change();');
        window.location.reload();
    };

    /*! \brief <a href="connection__01_8js_source.html">Subscribing topics</a> */
    /* ===== Subscribing topics ===== */ 
    function subscribe(connection) {
        connection.subscribe(function(msg) {
            map_o.callBackLas(msg);
        }, '/scan', 'sensor_msgs/LaserScan', 1);
        
        connection.subscribe(function(msg) {
            map_o.callBackPos(msg);
        }, '/move_base/local_costmap/robot_footprint', 'geometry_msgs/PolygonStamped', 1);
        
        connection.subscribe(function(msg) {
            map_o.callBackMap(msg);
            
        }, '/map', 'nav_msgs/OccupancyGrid', 1);
        connection.subscribe(function(msg) {
            map_o.callBackObs(msg);           
        }, '/move_base/local_costmap/obstacles', 'nav_msgs/GridCells', 1);
        // listener of position of laser
        connection.subscribe(function(msg) {
            map_o.callBackLaserPos(msg);           
        }, '/laser_web_tf', 'geometry_msgs/Pose', 1);
    }

    /*! \brief <a href="connection__01_8js_source.html">Unsubscribing topics</a> */
    /* ===== Unsubscribing topics ===== */ 
    function unsubscribe(connection) {
        connection.unsubscribe('/scan_filtered');
        connection.unsubscribe('/move_base/local_costmap/robot_footprint');
        connection.unsubscribe('/map');
        connection.unsubscribe('/move_base/local_costmap/obstacles');
    }

    /*! \brief <a href="connection__01_8js_source.html">Initialization assigning IP addresses</a> */
    /* ===== Initialization assigning IP addresses ===== */
    this.initIP = function() {
        console.log("Inicializace ip adresy");
        // Add element for ip assigning
        formIP = document.createElement('form');
        formIP.id = "ip_adr";
        formIP.setAttribute('action', 'javascript:con.ip_change();');

        var textIP = document.createElement('input');
        textIP.type = "text";
        textIP.id = "ip_adr_value";
        textIP.value = "localhost";// localhost//192.168.2.76
        textIP.name = "ip_adr_value";

        subIP = document.createElement('input');
        subIP.type = "submit";
        subIP.id = "ip_adr_ok";
        subIP.value = "connect";
        subIP.name = "ip_adr_ok";

        var address_div = document.getElementById('address');

        formIP.appendChild(textIP);
        formIP.appendChild(subIP);
        address_div.appendChild(formIP);
    };
}