/*! 
 \file laser_01.js
 \author Martin Blaha, Michal Krec, Tadeas Lejsek, Petr Marek, Tomas Nouza
 \brief Laser
 \details Representation of maps and data from the laser
 */

/*! \brief <a href="laser__01_8js_source.html">MapaLaser</a> */
/* ===== MapaLaser ===== */
function MapaLaser(connection, laser_div) {

    var w, h, size;
    var offsetX = 10;
    var offsetY = 10;
    var data;
    var zoomY = 1;
    var zoomX = 1;
    var resolution; // m/cell
    var canvasSizeX = 300;
    var canvasSizeY = 300;
    var laserPozX = 0;
    var laserPozY = 0;
    var laserOriZ = 0;
    var laserOriX = 0;
    var laserDX = 0;
    var laserDY = 0;
    var laser_enable;
    var map_enable;
    var obs_enable;
    var pos_enable;
    var button_properties;
    var table_properties;
    var matrix_properties_name = ['width', 'height'];
    var matrix_properties_value = ['300', '300'];
    var enable_Laser_draw = true;
    var enable_map_draw = true;
    var enable_obs_draw = true;
    var enable_pos_draw = true;

    var obal_div;
    var buffer_map, buffer_obs, buffer_las, buffer_pos;
    var canvas_map, canvas_obs, canvas_las, canvas_pos;
    var bcontext_map, bcontexc_obs, bcontext_las, bcontext_pos;
    var ccontext_map, ccontext_obs, ccontext_las, ccontext_pos;

    /*! \brief <a href="laser__01_8js_source.html">Initialization of laser</a> */
    /* ===== Initialization of laser ===== */
    this.init = function() {

        initForProp();
    };

    function initForProp() {
        while (laser_div.hasChildNodes()) {
            laser_div.removeChild(laser_div.lastChild);
        }

        var nameDiv = document.createElement("p");
        nameDiv.id = 'text_modeLaser';
        var nameTextMode = document.createTextNode('Laser');
        nameDiv.appendChild(nameTextMode);
        laser_div.appendChild(nameDiv);

        obal_div = document.createElement('div');
        obal_div.id = "obalLaserDiv";
        laser_div.setAttribute("width", (canvasSizeX + 100).toString());
        laser_div.setAttribute("height", (canvasSizeY + 100).toString());

        // Add enable laser drawing 
        laser_enable = document.createElement("input");
        laser_enable.type = "submit";
        laser_enable.id = 'laser_enable';
        laser_enable.value = 'laser enabled';
        laser_div.appendChild(laser_enable);

        // Add enable map drawing 
        map_enable = document.createElement("input");
        map_enable.type = "submit";
        map_enable.id = 'map_enable';
        map_enable.value = 'map enabled';
        laser_div.appendChild(map_enable);

        // Add enable obsatcles drawing 
        obs_enable = document.createElement("input");
        obs_enable.type = "submit";
        obs_enable.id = 'obs_enable';
        obs_enable.value = 'obstacles enabled';
        laser_div.appendChild(obs_enable);

        // Add enable obsatcles drawing 
        pos_enable = document.createElement("input");
        pos_enable.type = "submit";
        pos_enable.id = 'pos_enable';
        pos_enable.value = 'footprint enabled';
        laser_div.appendChild(pos_enable);

        // Add Properties button
        button_properties = document.createElement("input");
        button_properties.type = "submit";
        button_properties.id = 'laser_prop';
        button_properties.value = 'Properties';
        laser_div.appendChild(button_properties);

        // Add event lisener
        laser_enable.addEventListener('click', change_laser_enable, false);
        map_enable.addEventListener('click', change_map_enable, false);
        obs_enable.addEventListener('click', change_obs_enable, false);
        pos_enable.addEventListener('click', change_pos_enable, false);
        button_properties.addEventListener('click', change_prop, false);

        canvas_map = document.createElement('canvas');
        canvas_map.id = 'map_pic';
        canvas_map.setAttribute("width", canvasSizeX.toString());
        canvas_map.setAttribute("height", canvasSizeY.toString());
        canvas_map.style.position = 'absolute';
        canvas_map.style.zIndex = '0';
        obal_div.appendChild(canvas_map);
        laser_div.appendChild(obal_div);

        canvas_obs = document.createElement('canvas');
        canvas_obs.id = 'obs_pic';
        canvas_obs.setAttribute("width", canvasSizeX.toString());
        canvas_obs.setAttribute("height", canvasSizeY.toString());
        canvas_obs.style.position = 'absolute';
        canvas_obs.style.zIndex = '1';
        obal_div.appendChild(canvas_obs);
        laser_div.appendChild(obal_div);

        canvas_las = document.createElement('canvas');
        canvas_las.id = 'laser_pic';
        canvas_las.setAttribute("width", canvasSizeX.toString());
        canvas_las.setAttribute("height", canvasSizeY.toString());
        canvas_las.style.position = 'absolute';
        canvas_las.style.zIndex = '2';
        obal_div.appendChild(canvas_las);
        laser_div.appendChild(obal_div);

        canvas_pos = document.createElement('canvas');
        canvas_pos.id = 'pos_pic';
        canvas_pos.setAttribute("width", canvasSizeX.toString());
        canvas_pos.setAttribute("height", canvasSizeY.toString());
        canvas_pos.style.position = 'absolute';
        canvas_pos.style.zIndex = '3';
        obal_div.appendChild(canvas_pos);
        laser_div.appendChild(obal_div);

        //canvas = document.getElementById('laser_pic');
        if (!canvas_map || !canvas_obs || !canvas_las || !canvas_pos) {
            alert('canvas not found error');
            return;
        }

        if (!canvas_map.getContext || !canvas_obs.getContext || !canvas_las.getContext || !canvas_pos.getContext) {
            alert('getContext error');
            return;
        }

        // Get the 2D canvas context.
        ccontext_map = canvas_map.getContext('2d');
        ccontext_obs = canvas_obs.getContext('2d');
        ccontext_las = canvas_las.getContext('2d');
        ccontext_pos = canvas_pos.getContext('2d');

        if (!ccontext_map || !ccontext_obs || !ccontext_las || !ccontext_pos) {
            alert('getContext error');
            return;
        }

        // centering relative position
        w = canvas_map.width = canvas_map.clientWidth;
        h = canvas_map.height = canvas_map.clientHeight;

        document.getElementById("laser").style.width = '"' + w + '"';
        pom = h + 50;
        document.getElementById("laser").style.height = '"' + pom + '"';

        size = (w > h) ? h : w;

        canvas_obs.width = w;
        canvas_obs.height = h;
        canvas_las.width = w;
        canvas_las.height = h;
        canvas_pos.width = w;
        canvas_pos.height = h;

        buffer_map = document.createElement("canvas");
        bcontext_map = buffer_map.getContext('2d');
        buffer_map.width = w;
        buffer_map.height = h;

        buffer_obs = document.createElement("canvas");
        bcontexc_obs = buffer_obs.getContext('2d');
        buffer_obs.width = w;
        buffer_obs.height = h;

        buffer_las = document.createElement("canvas");
        bcontext_las = buffer_las.getContext('2d');
        buffer_las.width = w;
        buffer_las.height = h;

        buffer_pos = document.createElement("canvas");
        bcontext_pos = buffer_pos.getContext('2d');
        buffer_pos.width = w;
        buffer_pos.height = h;
    }

    /*! \brief <a href="laser__01_8js_source.html">Button listener for changing properties</a> */
    /* ===== Button listener for changing properties ===== */
    var change_prop = function(e) {
        console.log("change properties laser");
        // Remove previous childnodes 
        while (obal_div.hasChildNodes()) {
            obal_div.removeChild(obal_div.lastChild);
        }

        // Table with properties
        table_properties = document.createElement("table"); // Add table
        table_properties.border = "1";
        var tbody = document.createElement("tbody"); // Add body
        table_properties.appendChild(tbody);
        for (var j = 0; j < matrix_properties_name.length; j++) { // Count of rows
            var row = document.createElement("tr"); // Add row
            for (var k = 0; k < 2; k++) { // Count of columns
                var cell = document.createElement("td"); // Add cell
                if (k === 0) { // Name of property
                    data = document.createTextNode(matrix_properties_name[j]);
                }
                if (k === 1) { // Value of property
                    data = document.createElement("input");
                    data.type = "text";
                    data.name = "table_properties" + j;
                    data.id = "table_properties_Laser_" + matrix_properties_name[j];
                    data.value = matrix_properties_value[j];
                    obal_div.appendChild(data);
                }
                cell.appendChild(data);
                row.appendChild(cell);
            }
            tbody.appendChild(row);
        }
        obal_div.appendChild(table_properties);

        // OK button
        var ok_button = document.createElement("input");
        ok_button.type = "submit";
        ok_button.name = 'okPropLaser';
        ok_button.id = 'okPropLaser';
        ok_button.value = 'OK';
        obal_div.appendChild(ok_button);

        ok_button.addEventListener('click', function(msg) {
            canvasSizeX = parseFloat(document.getElementById('table_properties_Laser_width').value);
            canvasSizeY = parseFloat(document.getElementById('table_properties_Laser_height').value);
            matrix_properties_value[0] = canvasSizeX;
            matrix_properties_value[1] = canvasSizeY;
            initForProp();
        }, false);

        // CANCEL button
        var cancel_button = document.createElement("input");
        cancel_button.type = "submit";
        cancel_button.name = 'caPropLaser';
        cancel_button.id = 'caPropLaser';
        cancel_button.value = 'Cancel';
        obal_div.appendChild(cancel_button);

        cancel_button.addEventListener('click', function(msg) {
            // Remove previous childnodes 
            while (obal_div.hasChildNodes()) {
                obal_div.removeChild(obal_div.lastChild);
            }

            //camera_pic.appendChild(lastPic);
        }, false);
    };

    var change_laser_enable = function(e) {
        if (enable_Laser_draw) {
            enable_Laser_draw = false;
            console.log("change enable laser " + enable_Laser_draw);
            laser_enable.value = 'laser disabled';
            ccontext_las.clearRect(0, 0, w, h);
        } else {
            enable_Laser_draw = true;
            console.log("change enabled laser " + enable_Laser_draw);
            laser_enable.value = 'laser enabled';
            ccontext_las.drawImage(buffer_las, 0, 0);
        }
        console.log("change enabled laser");
    };


    var change_map_enable = function(e) {
        if (enable_map_draw) {
            enable_map_draw = false;
            console.log("change enable map " + enable_map_draw);
            map_enable.value = 'map disabled';
            ccontext_map.clearRect(0, 0, w, h);
        } else {
            enable_map_draw = true;
            console.log("change enable map " + enable_map_draw);
            map_enable.value = 'map enabled';
            ccontext_map.drawImage(buffer_map, 0, 0);
        }
        console.log("change enable map");
    };

    var change_obs_enable = function(e) {
        if (enable_obs_draw) {
            enable_obs_draw = false;
            console.log("change enable obstacles " + enable_obs_draw);
            obs_enable.value = 'obstacles disabled';
            ccontext_obs.clearRect(0, 0, w, h);
        } else {
            enable_obs_draw = true;
            console.log("change enable obsatcles " + enable_obs_draw);
            obs_enable.value = 'obsatcles enabled';
            ccontext_obs.drawImage(buffer_obs, 0, 0);
        }
        console.log("change enable obstacles");
    };

    var change_pos_enable = function(e) {
        if (enable_pos_draw) {
            enable_pos_draw = false;
            console.log("change enable position " + enable_pos_draw);
            pos_enable.value = 'position disabled';
            ccontext_pos.clearRect(0, 0, w, h);
        } else {
            enable_pos_draw = true;
            console.log("change enable position " + enable_pos_draw);
            pos_enable.value = 'position enabled';
            ccontext_pos.drawImage(buffer_pos, 0, 0);
        }
        console.log("change enable position");
    };

    this.callBackMap = function(msg) {
        drawMap(msg, '#000000');
    };

    this.callBackObs = function(msg) {
        drawObs(msg, '#0000ff');
    };
    this.callBackLaserPos = function(msg) {
        laserPozX = msg.position.x;
        laserPozY = msg.position.y;
        laserOriZ = msg.orientation.z;
        laserOriX = msg.orientation.x;

    };


    function drawMap(map, color) {
        var i;

        var width = map.info.width;
        var height = map.info.height;

        var max = (width < height) ? height : width;
        // set xzoom and yzoom
        zoomX = (canvasSizeX - 2 * offsetX) / (max);
        zoomY = (canvasSizeY - 2 * offsetY) / (max);
        resolution = map.info.resolution;

        laserDX = map.info.origin.position.x;
        laserDY = map.info.origin.position.y;

        bcontext_map.clearRect(0, 0, w, h);
        bcontext_map.beginPath();
        bcontext_map.fillStyle = color;

        for (i = 0; i < map.data.length; i++) {
            var map_x = i % width;
            var map_y = i / width;
            var hod = map.data[i];
            if (hod > 10) {
                bcontext_map.fillRect(zoomX * map_x + offsetX, canvasSizeY - (zoomY * map_y + offsetY), 2, 2);
            }
        }
        bcontext_map.stroke();
        if (enable_map_draw) {
            ccontext_map.clearRect(0, 0, w, h);
            ccontext_map.drawImage(buffer_map, 0, 0);
        }
    }

    function drawObs(obstacles, color) {
        var i;
        bcontexc_obs.clearRect(0, 0, w, h);
        bcontexc_obs.beginPath();
        bcontexc_obs.fillStyle = color;
        for (i = 0; i < obstacles.cells.length; i++) {
            var obs_x = obstacles.cells[i].x;
            var obs_y = obstacles.cells[i].y;
            bcontexc_obs.fillRect(zoomX * 1 / resolution * obs_x + offsetX, canvasSizeY - (zoomY * 1 / resolution * obs_y + offsetY), 2, 2);
        }
        bcontexc_obs.fillStyle = '#000000';
        bcontexc_obs.stroke();
        if (enable_obs_draw) {
            ccontext_obs.clearRect(0, 0, w, h);
            ccontext_obs.drawImage(buffer_obs, 0, 0);
        }
    }

    this.callBackLas = function(msg) {
  
        drawLas(msg, '#00ff00');
    };

    function sign(x) {
        return x > 0 ? 1 : x < 0 ? -1 : 0;
    }
    function drawLas(msg, color) {
        var min_uhel = msg.angle_min;
        var prirustek = msg.angle_increment;
        var num_ranges = msg.ranges.length;
        var i = 0;
       
       /* if (laserOriX < -(Math.PI - 0.05) || laserOriX > (Math.PI - 0.05)) */{
            bcontext_las.clearRect(0, 0, w, h);
            bcontext_las.beginPath();
            bcontext_las.fillStyle = color;

            for (i = 0; i <= num_ranges; i++) {
                var r = msg.ranges[i];

                var yawLaser = laserOriZ + sign(Math.cos(laserOriX)) * (min_uhel + i * prirustek);

                var xZed = laserPozX - laserDX + r * Math.cos(yawLaser);
                var yZed = laserPozY - laserDY + r * Math.sin(yawLaser);
                bcontext_las.fillRect(zoomX * 1 / resolution * xZed + offsetX, canvasSizeX - (zoomY * 1 / resolution * yZed + offsetY), 2, 2);
                // draw information 
                //bcontext_las.lineTo(zoomX * 1 / resolution * (laserPozX - laserDX) + offsetX, canvasSizeY - (zoomY * 1 / resolution * (laserPozY - laserDY) + offsetY));
                bcontext_las.fillStyle = '#ff0000';
                bcontext_las.fillRect(zoomX * 1 / resolution * (laserPozX - laserDX) + offsetX, canvasSizeY - (zoomY * 1 / resolution * (laserPozY - laserDY) + offsetY), 2, 2);
                bcontext_las.fillStyle = color;
                //  bcontext_las.lineTo(zoomX * 1 / resolution * (laserPozX + 1 * Math.cos(laserOriZ)) + offsetX, canvasSizeY - (zoomY * 1 / resolution * (laserPozY + 1 * Math.sin(laserOriZ)) + offsetY));

            }

            bcontext_las.stroke();
            if (enable_Laser_draw) {
                ccontext_las.clearRect(0, 0, w, h);
                ccontext_las.drawImage(buffer_las, 0, 0);
            }
        }
    }


    this.callBackPos = function(msg) {

        drawPos(msg, '#000000');
    };

    function drawPos(msg, color) {
        var c;
        //bcontex_pos.clearRect(0, 0, m_context.canvas.width, m_context.canvas.height);
        bcontext_pos.clearRect(0, 0, w, h);
        bcontext_pos.beginPath();
        bcontext_pos.lineWidth = 2;
        bcontext_pos.strokeStyle = color;
        for (c = 0; c < msg.polygon.points.length - 1; c++) {
            bcontext_pos.lineTo(zoomX * 1 / resolution * msg.polygon.points[c].x + offsetX, canvasSizeY - (zoomY * 1 / resolution * msg.polygon.points[c].y + offsetY));
            bcontext_pos.lineTo(zoomX * 1 / resolution * msg.polygon.points[c + 1].x + offsetX, canvasSizeY - (zoomY * 1 / resolution * msg.polygon.points[c + 1].y + offsetY));

        }

        var firstPY = msg.polygon.points[0].y;
        var lastPY = msg.polygon.points[c].y;
        var firstPX = msg.polygon.points[0].x;
        var lastPX = msg.polygon.points[c].x;


        bcontext_pos.lineTo(zoomX * 1 / resolution * lastPX + offsetX, canvasSizeY - (zoomY * 1 / resolution * lastPY + offsetY));
        bcontext_pos.lineTo(zoomX * 1 / resolution * firstPX + offsetX, canvasSizeY - (zoomY * 1 / resolution * firstPY + offsetY));

        bcontext_pos.stroke();

        if (enable_pos_draw) {
            ccontext_pos.clearRect(0, 0, w, h);
            ccontext_pos.drawImage(buffer_pos, 0, 0);
        }
    }
}